# -*- coding: utf-8 -*-

# Almost all code comes from http://code.activestate.com/recipes/552751/

import sys
import json
import select 
import threading
import traceback
import socketserver
import xmlrpc.server
try:
    import fcntl
except ImportError:
    fcntl = None


class SimpleJSONRPCDispatcher(xmlrpc.server.SimpleXMLRPCDispatcher):
    def _marshaled_dispatch(self, request_raw, dispatch_method = None):
        id = None
        try:
            request = json.loads(request_raw.decode("utf-8"))
            method = request['method']
            params = request['params']
            id     = request['id']

            if dispatch_method is not None:
                result = dispatch_method(method, params)
            else:
                result = self._dispatch(method, params)
            response = dict(id=id, result=result, error=None)
        except:
            extpe, exv, extrc = sys.exc_info()
            err = dict(type=str(extpe),
                       message=str(exv),
                       traceback=''.join(traceback.format_tb(extrc)))
            response = dict(id=id, result=None, error=err)
        try:
            response_raw = json.dumps(response, ensure_ascii=False)
            if sys.version_info[1] != 0:
                response_raw = response_raw.encode("utf-8")
            return response_raw
        except:
            extpe, exv, extrc = sys.exc_info()
            err = dict(type=str(extpe),
                       message=str(exv),
                       traceback=''.join(traceback.format_tb(extrc)))
            response = dict(id=id, result=None, error=err)
            
            response_raw = json.dumps(response, ensure_ascii=False)
            if sys.version_info[1] != 0:
                response_raw = response_raw.encode("utf-8")
            return response_raw


class SimpleJSONRPCRequestHandler(xmlrpc.server.SimpleXMLRPCRequestHandler):
    # Class attribute listing the accessible path components;
    # paths not on this list will result in a 404 error.
    rpc_paths = ('/', '/JSON')


class SimpleJSONRPCServer(socketserver.TCPServer,
                          SimpleJSONRPCDispatcher):
    """Simple JSON-RPC server.

    Simple JSON-RPC server that allows functions and a single instance
    to be installed to handle requests. The default implementation
    attempts to dispatch JSON-RPC calls to the functions or instance
    installed in the server. Override the _dispatch method inhereted
    from SimpleJSONRPCDispatcher to change this behavior.
    """

    allow_reuse_address = True

    def __init__(self, addr, requestHandler=SimpleJSONRPCRequestHandler,
                 logRequests=True):
        self.logRequests = logRequests

        SimpleJSONRPCDispatcher.__init__(self, allow_none=True, encoding=None)
        socketserver.TCPServer.__init__(self, addr, requestHandler)

        self.__thread = None  

    def serve_forever(self, in_thread=False, poll_interval=0.5):
        def serve_thread(server, poll_interval):
            server.serve_forever(poll_interval=poll_interval)
        if in_thread:
            args = [self, poll_interval]
            self.__thread = threading.Thread(target=serve_thread, args=args)
            self.__thread.start()
        else:
            socketserver.TCPServer.serve_forever(self, poll_interval)

    def shutdown(self, immediately = True):
        if not immediately:
            self._BaseServer__serving = False
            return
        socketserver.TCPServer.shutdown(self)
        if self.__thread:
            self.__thread.join()
            self.__thread = None



# -*- coding: utf-8 -*-

"""Commitment oriented architecture framework implementations.

This is my experimental idea.

In Hollywood's raw, object B doesn't know when it called(System A).

A -> B

In This framework, it has different philosophy(System B).

A ... I'm hungry
  B ... Yes! Have a humbarger. Here you are!

System A is allopoietic and system B is autopoietic(but it has no special
features to generate copies). All objects in system B behave distributed
autonomously. Its object knows boundary between itself and outer world.
It knows what it can do and when it should work. In Hollywood's raw,
It knows what it can do but it doesn't know when it should work.

This implementation's API uses metaphor of twitter(but it does not follows
people but message/event):

  class A:
    def do_something(self):
      twitter("I'm hungry")

  class B(__metaclass__=Follower):
    @following("I'm hungry")
    def give_hunberger(self, message):
      print("Yes! Have a humbarger. Here you are!")

Recent frameworks inject dependencies in setting up time. But all objects
in this framework don't have dependencies at any time. In fact, you should use
this mechanism between same level objects. I call it "society".
Many systems (including real world) are hierarchical society.
Any books says that 200 is the biggest members it can live in one group
without any special rules and hierarchy.
Between societies, you can use normal relationship(including Hallywoods raw).

Have fun!

This architecture inspired by Toradora!. It's Japanese nobel/cartoon.
Characters in this pieces behave to give profit for others.
They live strongly.
This story is more impressed than Takeshi Kitano's films for me.

"""

import re
import sys
import queue
import types
import weakref
import itertools
import functools


def _dummy(message):
    """Dummy guard condition function.

    It always matches.

    @param message: passed values from expositions.
    @type  message: Message object.
    @return: whether follower should be called or not.
    @rtype : boolean.
    """
    return True


class Identifier:
    """Event naming identifier.

    identifier is built by string.
    It supports wildcard.

    "sample:*" => match any action in "sample"
    "*:entry" => match any "entry" messsge

    @future: including URI(for external followable node)
    """
    __slots__ = ("name", "action", "namespace", "functions")
    _split_name_action = re.compile(r"(.*):(.*)")
    _split_namespace = re.compile(r"(.*)@(.*)")

    def __init__(self, key_or_identifier, action=None, guard_condition=_dummy):
        self.functions = [None, None]
        if action is not None:
            self.name = key_or_identifier.name
            self.namespace = key_or_identifier.namespace
            self.action = set([action])
            self.functions[0] = self._match_all
            return
        name, namespace, action = self._split_key(key_or_identifier)
        self.namespace = namespace
        if name == "*" and action == "*":
            self.name = None
            self.action = None
            self.functions[0] = self._match_always
        elif name == "*" and action != "*":
            self.name = None
            self.action = set([action])
            self.functions[0] = self._match_action_only
        elif name != "*" and action == "*":
            self.name = name
            self.action = None
            self.functions[0] = self._match_name_only
        elif name != "*" and action != "*":
            self.name = name
            self.action = set([action])
            self.functions[0] = self._match_all
        self.functions[1] = guard_condition

    def _split_key(self, key):
        match = self._split_name_action.match(key)
        if match:
            name, action = match.groups()
        else:
            name = key
            action = "*"
        match = self._split_namespace.match(name)
        if match:
            name, namespace = match.groups()
        else:
            namespace = None
        return name, namespace, action

    def __str__(self):
        return "<berrymq.Identifier object at %d: id=%s>" % (
            id(self), self.id_str())

    def id_str(self):
        name = "*" if self.name is None else self.name
        action = "*" if self.action is None else ",".join(sorted(self.action))
        return "%s:%s" % (name, action)

    def _match_all(self, rhs):
        return self._match_name_only(rhs) and self._match_action_only(rhs)

    def _match_name_only(self, rhs):
        if rhs.name is None:
            return True
        return self.name == rhs.name

    def _match_action_only(self, rhs):
        if rhs.action is None:
            return True
        return bool(self.action & rhs.action)

    def _match_always(self, rhs):
        return True

    def is_match(self, expose_identifier, message=None):
        if self.functions[0](expose_identifier):
            return self.functions[1](message)
        return False

    def is_local(self):
        return self.name.startswith("_")


class cond(object):
    def __init__(self, identifier, guard_condition=_dummy):
        self.targets = [Identifier(identifier, guard_condition=guard_condition)]

    def __and__(self, rhs):
        newtarget = target(None, None)
        newtarget.targets = self.targets + rhs.targets
        return newtarget


def _get_attributes(instance, typeobj):
    """Helper generator to implement TDFW.

    It returns match objects.
    """
    for key in dir(instance):
        if key.startswith("__"):
            continue
        value = getattr(instance, key)
        if type(value) == typeobj:
            yield value


class _weakmethod_ref:
    """This method is arange version of Python Cookbook 2nd 6.10"""
    __slots__ = ("_obj", "_func")
    def __init__(self, fn):
        try:
            o, f = fn.__self__, fn.__func__
        except AttributeError:
            self._obj = None
            self._func = fn
        else:
            self._obj = weakref.ref(o)
            self._func = f

    def __call__(self):
        if self._obj is None:
            return self._func
        elif self._obj() is None:
            return None
        return types.MethodType(self._func, self._obj())


class _weakfunction_ref:
    """This method is arange version of Python Cookbook 2nd 6.10"""
    __slots__ = ("_func")
    def __init__(self, fn):
        self._func = weakref.ref(fn)

    def __call__(self):
        if self._func() is None:
            return None
        return self._func()


class Transporter:
    _followers = {}

    def get_valid_followers(self):
        for name, followers in self._followers.items():
            for id_obj, function_wrapper in followers:
                function = function_wrapper()
                if function is not None:
                    yield id_obj, function

    def regist_follower(self, id_obj, function):
        followers = self._followers.setdefault(id_obj.name, [])
        followers.append((id_obj, function))

    def twitter(self, id_obj, args, kwargs, counter=100):
        message = Message(id_obj, args, kwargs, counter)
        if id_obj.name is None:
            actions = self._followers.values()
        else:
            wildcard_actions = self._followers.get(None, [])
            certain_actions = self._followers.get(id_obj.name, [])
            actions = [wildcard_actions, certain_actions]
        for following_id, function in itertools.chain(*actions):
            if not following_id.is_match(id_obj, message):
                continue
            if function() is None:
                continue # delete after
            else:
                function()(message)


class RootTransporter:
    _default_namespace = None
    _namespaces = {}

    @classmethod
    def twitter(cls, id_obj, args, kwargs):
        namespace = id_obj.namespace
        if namespace is None:
            namespace = cls._default_namespace
        cls.get(namespace).twitter(id_obj, args, kwargs)

    @classmethod
    def regist_follower(cls, id_obj, method):
        namespace = id_obj.namespace
        if namespace is None:
            namespace = cls._default_namespace
        cls.get(namespace).regist_follower(id_obj, method)

    @classmethod
    def show_followers(cls):
        pass

    @classmethod
    def get(cls, namespace):
        message_queue = cls._namespaces.get(namespace)
        if message_queue is None:
            message_queue = Transporter()
            cls._namespaces[namespace] = message_queue
        return message_queue

    @classmethod
    def request_work(cls, target_function, message, command="process"):
        cls._qin.put((command, target_function, message))

    @classmethod
    def get_all_errors(cls):
        return cls._get_all_from_queue(cls._qerr)

    @classmethod
    def stop_thread_pool(cls):
        for i in range(len(cls._pool)):
            cls.request_work(None, None, "stop")
        for existing_thread in cls._pool:
            existing_thread.join()
        del cls._pool[:]


class MethodDecorator:
    """This class is attached to exposition/following functions/methods.

    After initialization, this shows any information to draw connections.

    @sa exposition
    @sa following
    @sa following_method
    """
    __slots__ = ("expositions", "followers", "is_init")
    attribute_name = "__exposition_information"
    def __init__(self):
        self.followers = []
        self.is_init = False

    @classmethod
    def add_follower(cls, id_obj, function):
        attribute = cls._set_open_method_attribute(function)
        attribute.followers.append(id_obj)

    @classmethod
    def _set_open_method_attribute(cls, function):
        attribute = getattr(function, cls.attribute_name, None)
        if attribute is None:
            attribute = cls()
            setattr(function, cls.attribute_name, attribute)
        return attribute


def following_function(identifier, guard_condition=_dummy):
    id_obj = Identifier(identifier, guard_condition=guard_condition)
    def _(f):
        RootTransporter.regist_follower(id_obj, _weakfunction_ref(f))
        return f
    return _


def following(identifier, guard_condition=_dummy):
    id_obj = Identifier(identifier, guard_condition=guard_condition)
    def _(f):
        MethodDecorator.add_follower(id_obj, f)
        return f
    return _


def auto_twitter(identifier, entry=False, exit=False):
    id_obj = Identifier(identifier)
    def _(func):
        if entry == exit:
            entry_id = Identifier(id_obj, "entry")
            exit_id = Identifier(id_obj, "exit")
            @functools.wraps(func)
            def __(*args, **kwargs):
                RootTransporter.twitter(entry_id, args, kwargs)
                func(*args, **kwargs)
                RootTransporter.twitter(exit_id, args, kwargs)
            return __
        elif exit or id_obj.action is not None and "exit" in id_obj.action:
            id_obj.action = set(["exit"])
            @functools.wraps(func)
            def __(*args, **kwargs):
                func(*args, **kwargs)
                RootTransporter.twitter(id_obj, args, kwargs)
            return __
        elif entry or id_obj.action is not None and "entry" in id_obj.action:
            id_obj.action = set(["entry"])
            @functools.wraps(func)
            def __(*args, **kwargs):
                RootTransporter.twitter(id_obj, args, kwargs)
                func(*args, **kwargs)
            return __
        return func
    return _


class Follower(type):
    """Metaclass for definition of follower instance methods.

    use like this:

      class Logger(metaclass=Follower):
          @following_method("function", "call")
          def log_function_call(self, message):
              ...

          @classmethod
          @following_classmethod("error", "raised")
          def show_error(cls, message):
              ...

          @staticmethod
          @following("thread", "created")
          def log_thread(message):
              ...
    """
    def __new__(cls, name, bases, dict):
        """Create exposition point for classmethod."""
        newtype = type.__new__(cls, name, bases, dict)
        for method in _get_attributes(newtype, types.MethodType):
            attribute = getattr(method, MethodDecorator.attribute_name, None)
            if attribute is None:
                continue
            for id_obj in attribute.followers:
                RootTransporter.regist_follower(id_obj, _weakmethod_ref(method))
            attribute.is_init = True
        return newtype

    def __call__(cls, *args):
        """Create exposition point for instancemethod."""
        instance = type.__call__(cls, *args)
        for method in _get_attributes(instance, types.MethodType):
            attribute = getattr(method, MethodDecorator.attribute_name, None)
            if attribute is None:
                continue
            if attribute.is_init:
                continue
            for id_obj in attribute.followers:
                RootTransporter.regist_follower(id_obj, _weakmethod_ref(method))
        return instance


class Message:
    def __init__(self, id_obj, args, kwargs, counter):
        self._id_obj = id_obj
        self._args = args
        self._kwargs = kwargs
        self._counter = counter
        self.__dict__.update(kwargs)

    @property
    def name(self):
        return self._id_obj._name

    @property
    def action(self):
        return self._id_obj._action

    @property
    def args(self):
        return self._args

    @property
    def kwargs(self):
        return self._kwargs

    @property
    def id(self):
        return self._id_obj.id_str()

    def twitter(self, id_obj, **kwargs):
        RootTransporter.twitter(id_obj, kwargs, self._counter-1)

    def apply(self, func):
        """Utility method to enease method call deligation.

        You can use like this:
          @exposition("result", "show")
          def show_result(score):
              print("my score:", score)
              # get 2 kwparams
              twitter("result", "show", username="shibu", score=score) 

          def log(username, score): # same params with exposition point.
              ...

          @following("result", "show")
          def log_score_exposition(message):
              message.apply(log) # use this!

        """
        return func(**self.kwargs)


def show_followers():
    methods = []
    for id_obj, function_wrapper in RootTransporter.get_valid_followers():
        for action in id_obj.action:
            methods.append("%s:%s" % (id_obj.name, action))
    return sorted(set(methods))


def twitter(identifier, *args, **kwargs):
    RootTransporter.twitter(Identifier(identifier), args, kwargs)



# -*- coding: utf-8 -*-

import time
import unittest
import berrymq.jsonrpc.server
import berrymq.jsonrpc.client


class TestJSONRPC(unittest.TestCase):
    port_bias = 0
    def setUp(self):
        server_url = ("localhost", 12345 + TestJSONRPC.port_bias)
        self.url = "http://localhost:%d" % (12345 + TestJSONRPC.port_bias)
        self.server = berrymq.jsonrpc.server.SimpleJSONRPCServer(server_url)

    def tearDown(self):
        self.server.shutdown()    
        TestJSONRPC.port_bias += 1

    def test_send_message(self):
        called = []
        def test_func():
            called.append("test_func called")
            return True
        self.server.register_function(test_func)
        self.server.serve_forever(in_thread=True)
        client = berrymq.jsonrpc.client.ServerProxy(self.url)
        client.test_func()
        self.assertEquals(["test_func called"], called)

    def test_reply(self):
        def test_func():
            return 123
        self.server.register_function(test_func)
        self.server.serve_forever(in_thread=True)
        client = berrymq.jsonrpc.client.ServerProxy(self.url)
        self.assertEquals(123, client.test_func())

    def test_invalid_parameters(self):
        self.server.register_function(lambda x,y:x+y, 'add')
        self.server.serve_forever(in_thread=True)
        client = berrymq.jsonrpc.client.ServerProxy(self.url)
        self.assertRaises(berrymq.jsonrpc.client.Fault, client.add, 5, "toto")

    def test_invalid_parameters_anity(self):
        self.server.register_function(lambda x,y:x+y, 'add')
        self.server.serve_forever(in_thread=True)
        client = berrymq.jsonrpc.client.ServerProxy(self.url)
        self.assertRaises(berrymq.jsonrpc.client.Fault, client.add, 5, 6, 7)

    def test_invalid_method_name(self):
        self.server.register_function(lambda x,y:x+y, 'add')
        self.server.serve_forever(in_thread=True)
        client = berrymq.jsonrpc.client.ServerProxy(self.url)
        self.assertRaises(berrymq.jsonrpc.client.Fault, client.addx, 2, 4)


def test_setup():
    test_suite = unittest.TestSuite()
    test_suite.addTest(unittest.makeSuite(TestJSONRPC))
    return test_suite



# -*- coding: utf-8 -*-
"""test program for interprocess communication(JSON-RPC layer)
"""

import os
import sys
import time
import unittest

rootdir = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
sys.path.insert(0, rootdir)
import berrymq
import berrymq.jsonrpc.server
import berrymq.jsonrpc.client
import berrymq.adapter.growl


"""
Test Sequence
=============

Use 2 nodes,  Primary(P) and Secondary(S).

1. Launch P
   Control JSONRPC Server:localhost:12345
2. Launch S
   Open JSON-RPC Server 
3. P <- S
   send "start_slyle02_test"
4. P -> S
   call connect_oneway()
5. P -> S
   call send_message("style02:test02", [1,2,3], {"a":1, "b":2})
6. P -> S
   return
7. P <- S
   twitter("style01:test01")
8. P <- S
   verify @ S
   send "exit_style01_test"
   verify @ P
9. P <- S
   send "start_stlye02_test"
10. P -> S
   close_connection() and wait 1[s]
11. P -> S
   connect_oneway()
12. 
"""

# Const

CONTROL_SERVER_URL = ("localhost", 12345) # primary node
PRIMARY_NODE_URL = ("localhost", 12346)
SECONDARY_NODE_URL = ("localhost", 12347)

# Utilities

def _url(URL):
    return "http://%s:%s" % URL

def check(expected, actual):
    if actual == expected:
        print("  ok: %s" % expected)
    else:
        print("  ng: expected=%s, acutal=%s" % (expected, actual))

def quit():
    print("quit")
    jsonserver.shutdown(immediately=False)


# Test Entry Points
# 
# These functions is run at primary node and called from secondary one via RPC.


class Style01Test(object):
    def __init__(self):
        self.received_messages = []

    def message_receiver(self, message):
        self.received_messages.append(message.id)

    def start(self):
        berrymq.regist_method("*", self.message_receiver)
        berrymq.init_connection(PRIMARY_NODE_URL)
        berrymq.interconnect(SECONDARY_NODE_URL)
        berrymq.twitter("style01c:test01")
        time.sleep(1)
        return True

    def exit(self):
        expected = ['style01c:test01', 'style01s:test02']
        check(expected, self.received_messages)
        berrymq.close_connection()
        return True


class Style02Test(object):
    def start(self):
        berrymq.connect_oneway(SECONDARY_NODE_URL)
        berrymq.send_message("style02c:test02", 1, 2, 3, a=1, b=2)
        return True

    def exit(self):
        berrymq.close_connection()
        return True


class Style03Test(object):
    def start(self):
        berrymq.connect_via_queue(SECONDARY_NODE_URL, "style03s:*")
        berrymq.send_message("style03c:test01", 3, 2, 1, a=1, b=2)
        return True

    def check(self):
        check("style03s:test02", berrymq.get().id)
        return True

    def exit(self):
        check("style03s:test03", berrymq.get_nowait().id)
        berrymq.close_connection()
        return True


class TestSuite(object):
    def __init__(self):
        self.style01 = Style01Test()
        self.style02 = Style02Test()
        self.style03 = Style03Test()


# Main Routines

def primary_node():
    global jsonserver
    jsonserver = berrymq.jsonrpc.server.SimpleJSONRPCServer(CONTROL_SERVER_URL)
    jsonserver.register_instance(TestSuite(), allow_dotted_names=True)
    jsonserver.register_function(quit)
    print("start primary server. waiting secondary node.")

    @berrymq.following_function("style03s:*")
    def receive_messages(message):
        _primary_node_test_result.append(message)
    
    berrymq.twitter("start primary server:info")
    jsonserver.serve_forever()


def secondary_node():
    expected_at_secondary = [
        ["style02c:test02", [1,2,3], {"a":1, "b":2}],
        ["style03c:test01", [3,2,1], {"a":1, "b":2}],
        ["style03s:test02", (), {}],
        ["style03s:test03", (), {}],
        ["style01c:test01", [], {}],
        ["style01s:test02", (), {}],
    ]
    test_results = []
    @berrymq.following_function("*:*")
    def test_receiver(message):
        expected = expected_at_secondary[0]
        if expected[0] == message.id and \
           expected[1] == message.args and \
           expected[2] == message.kwargs:
            result = "ok"
        else:
            result = "ng"
        del expected_at_secondary[0]
        test_results.append([str(expected), 
                             str([message.id, message.args, message.kwargs]), 
                             result])
    
    berrymq.init_connection(SECONDARY_NODE_URL)

    controller = berrymq.jsonrpc.client.ServerProxy(_url(CONTROL_SERVER_URL))
    controller.style02.start()
    controller.style02.exit()

    controller.style03.start()
    berrymq.twitter("style03s:test02")
    controller.style03.check()
    berrymq.twitter("style03s:test03")
    controller.style03.exit()

    controller.style01.start()
    berrymq.twitter("style01s:test02")
    time.sleep(1)

    controller.style01.exit()
    controller.quit()

    berrymq.close_connection()

    for expected, actual, result in test_results:
        if result == "ok":
            print("ok: expected = %s" % expected)
        else:
            print("ng: expected = %s, actual = %s" % (expected, actual))
    if len(test_results) == 0:
        print("ng: no message received")

def usage():
    print("""test JSON-RPC level inter process communication

    Run this scprit twice at same machine. First one is primary and second 
    is seconary.

usage:
    python test_interprocess_low.py -primary   [option]
         : run this first
    python test_interprocess_low.py -secondary [option]
         : run after primary process

option:
    -growl : transfer messages to growl(for debug)
""")

if __name__ == "__main__":
    if "-growl" in sys.argv:
        listner = berrymq.adapter.growl.GrowlAdapter("*:*")
    if "-primary" in sys.argv:
        primary_node()
    elif "-secondary" in sys.argv:
        secondary_node()
    else:
        usage()



#!/usr/bin/env python

"""Growl 0.6 Network Protocol Client for Python

modified by SHIBUKAWA Yoshiki to follow PEP-8 and remove deprecate warning.
"""
__version__ = "0.6" # will always match Growl version
__author__ = "Rui Carmo (http://the.taoofmac.com)"
__copyright__ = "(C) 2004 Rui Carmo. Code under BSD License."
__contributors__ = "Ingmar J Stein (Growl Team)"

import struct
from hashlib import md5
from socket import AF_INET, SOCK_DGRAM, socket

GROWL_UDP_PORT=9887
GROWL_PROTOCOL_VERSION=1
GROWL_TYPE_REGISTRATION=0
GROWL_TYPE_NOTIFICATION=1

class GrowlRegistrationPacket:
    """Builds a Growl Network Registration packet.
       Defaults to emulating the command-line growlnotify utility."""

    def __init__(self, application="growlnotify", password = None ):
        self.notifications = []
        self.defaults = [] # array of indexes into notifications
        self.application = application.encode("utf-8")
        self.password = password
    # end def


    def addNotification(self, notification="Command-Line Growl Notification", 
                        enabled=True):
        """Adds a notification type and sets whether it is enabled on the GUI"""
        self.notifications.append(notification)
        if enabled:
            self.defaults.append(len(self.notifications)-1)
    # end def


    def payload(self):
        """Returns the packet payload."""
        self.data = struct.pack( "!BBH",
                                 GROWL_PROTOCOL_VERSION,
                                 GROWL_TYPE_REGISTRATION,
                                 len(self.application) )
        self.data += struct.pack( "BB",
                                  len(self.notifications),
                                  len(self.defaults) )
        self.data += self.application
        for notification in self.notifications:
            encoded = notification.encode("utf-8")
            self.data += struct.pack("!H", len(encoded))
            self.data += encoded
        for default in self.defaults:
            self.data += struct.pack("B", default)
        self.checksum = md5()
        self.checksum.update(self.data)
        if self.password:
            self.checksum.update(self.password)
        self.data += self.checksum.digest()
        return self.data
    # end def
# end class


class GrowlNotificationPacket:
    """Builds a Growl Network Notification packet.
    Defaults to emulating the command-line growlnotify utility."""

    def __init__(self, application="growlnotify",
                notification="Command-Line Growl Notification", title="Title",
                description="Description", priority = 0, sticky = False, 
                password = None ):
        self.application  = application.encode("utf-8")
        self.notification = notification.encode("utf-8")
        self.title        = title.encode("utf-8")
        self.description  = description.encode("utf-8")
        flags = (priority & 0x07) * 2
        if priority < 0:
            flags |= 0x08
        if sticky:
            flags = flags | 0x0001
        self.data = struct.pack( "!BBHHHHH",
                                GROWL_PROTOCOL_VERSION,
                                GROWL_TYPE_NOTIFICATION,
                                flags,
                                len(self.notification),
                            len(self.title),
                            len(self.description),
                            len(self.application) )
        self.data += self.notification
        self.data += self.title
        self.data += self.description
        self.data += self.application
        self.checksum = md5()
        self.checksum.update(self.data)
        if password:
            self.checksum.update(password)
        self.data += self.checksum.digest()
    # end def

    def payload(self):
        """Returns the packet payload."""
        return self.data
    # end def
# end class


if __name__ == '__main__':
    print("Starting Unit Test")
    print(" - please make sure Growl is listening for network notifications")
    addr = ("localhost", GROWL_UDP_PORT)
    s = socket(AF_INET,SOCK_DGRAM)
    print("Assembling registration packet like growlnotify's (no password)")
    p = GrowlRegistrationPacket()
    p.addNotification()
    print("Sending registration packet")
    s.sendto(p.payload(), addr)
    import time
    time.sleep(3)

    print("Assembling standard notification packet")
    p = GrowlNotificationPacket()
    print("Sending standard notification packet")
    s.sendto(p.payload(), addr)

    print("Assembling priority -2 (Very Low) notification packet")
    p = GrowlNotificationPacket(priority=-2)
    print("Sending priority -2 notification packet")
    s.sendto(p.payload(), addr)

    print("Assembling priority 2 (Very High) sticky notification packet")
    p = GrowlNotificationPacket(priority=2,sticky=True)
    print("Sending priority 2 (Very High) sticky notification packet")
    s.sendto(p.payload(), addr)
    s.close()
    print("Done.")



# -*- coding: utf-8 -*-

import unittest
berrymq = None

class IdentifierTest(unittest.TestCase):
    def test_match(self):
        exposer = berrymq.Identifier("id_sample1:entry")
        follower = berrymq.Identifier("id_sample1:entry")
        self.assert_(follower.is_match(exposer))

    def test_match_only_name(self):
        exposer = berrymq.Identifier("id_sample2")
        follower = berrymq.Identifier("id_sample2:entry")
        self.assert_(follower.is_match(exposer))
    
    def test_wildcard_action(self):
        exposer = berrymq.Identifier("id_sample3:test")
        follower = berrymq.Identifier("id_sample3:*")
        self.assert_(follower.is_match(exposer))

    def test_wildcard_name_1(self):
        exposer = berrymq.Identifier("id_sample4:entry")
        follower = berrymq.Identifier("*:entry")
        self.assert_(follower.is_match(exposer))

    def test_wildcard_name_2(self):
        exposer = berrymq.Identifier("*:entry")
        follower = berrymq.Identifier("id_sample4:entry")
        self.assert_(follower.is_match(exposer))

    def test_wildcard_all_1(self):
        exposer = berrymq.Identifier("id_sample5:entry")
        follower = berrymq.Identifier("*:*") 
        self.assert_(follower.is_match(exposer))

    def test_wildcard_all_2(self):
        exposer = berrymq.Identifier("id_sample6:entry")
        follower = berrymq.Identifier("*") 
        self.assert_(follower.is_match(exposer))

    def test_local_namespace(self):
        exposer = berrymq.Identifier("_id_sample7:entry")
        self.assert_(exposer.is_local)

    def test_namespace(self):
        exposer = berrymq.Identifier("id_sample8@layer:entry")
        follower = berrymq.Identifier("*@layer:entry")
        self.assertEqual("layer", exposer.namespace)
        self.assertEqual("layer", follower.namespace)


class TestMessageSendAndReceive(unittest.TestCase):
    def test_received_by_function1(self):
        call_history = []
        @berrymq.auto_twitter("sample1", entry=True)
        def sample_expose1():
            call_history.append("expose")

        @berrymq.following_function("sample1:entry")
        def test_entry(message):
            self.assertEqual("sample1:entry", message.id) 
            call_history.append("entry")

        sample_expose1()
        self.assertEqual(["entry", "expose"], call_history)

    def test_received_by_function2(self):
        call_history = []
        @berrymq.auto_twitter("sample2", exit=True)
        def sample_expose():
            call_history.append("expose")

        @berrymq.following_function("sample2:exit")
        def test_entry(message):
            call_history.append("exit")

        sample_expose()
        self.assertEqual(["expose", "exit"], call_history)

    def test_auto_twitter(self):
        call_history = []
        @berrymq.auto_twitter("sample3")
        def sample_expose():
            call_history.append("expose")

        @berrymq.following_function("sample3:entry")
        def test_entry(message):
            call_history.append("entry")

        @berrymq.following_function("sample3:exit")
        def test_exit(message):
            call_history.append("exit")

        sample_expose()
        self.assertEqual(["entry", "expose", "exit"], call_history)

    def test_original_exposepoint(self):
        call_history = []

        def sample_expose3():
            call_history.append("expose")
            berrymq.twitter("sample4:original")

        @berrymq.following_function("sample4:original")
        def test_original(message):
            call_history.append("original")

        sample_expose3()
        self.assertEqual(["expose", "original"], call_history)

    def test_wildcard_following(self):
        call_history = []

        def sample_twitter3():
            call_history.append("action")
            berrymq.twitter("sample10:action")

        @berrymq.following_function("*:action")
        def test_original_action(message):
            call_history.append("accept")

        sample_twitter3()
        self.assertEqual(["action", "accept"], call_history)

    def test_staticmethod(self):
        call_history = []
        @berrymq.auto_twitter("sample5:entry")
        def sample_expose():
            call_history.append("expose")

        class TestClass(object):
            @staticmethod
            @berrymq.following_function("sample5:entry")
            def test_entry(message):
                call_history.append("entry")

        sample_expose()
        self.assertEqual(["entry", "expose"], call_history)

    def test_classmethod(self):
        call_history = []
        @berrymq.auto_twitter("sample6:entry")
        def sample_expose():
            call_history.append("expose")

        class TestClass(metaclass = berrymq.Follower):
            @classmethod
            @berrymq.following("sample6:entry")
            def test_entry(cls, message):
                call_history.append("entry")

        sample_expose()
        self.assertEqual(["entry", "expose"], call_history)

    def test_instancemethod(self):
        call_history = []
        @berrymq.auto_twitter("sample7:entry")
        def sample_expose():
            call_history.append("expose")

        class TestClass(metaclass = berrymq.Follower):
            @berrymq.following("sample7:entry")
            def test_entry(self, message):
                call_history.append("entry")
        instance = TestClass()
        sample_expose()
        self.assertEqual(["entry", "expose"], call_history)

    def test_guard_condition_test(self):
        call_history = []
        @berrymq.auto_twitter("sample8")
        def sample_expose(arg):
            call_history.append("expose")

        @berrymq.following_function("sample8:entry", 
            guard_condition=lambda message: message.args[0] == "match")
        def test_entry(message):
            call_history.append("entry")

        @berrymq.following_function("sample8:exit", 
            guard_condition=lambda message: message.args[0] == "match")
        def test_entry(message):
            call_history.append("exit")

        sample_expose("not match")
        self.assertEqual(["expose"], call_history)

    def test_wildcard(self):
        call_history = []
        @berrymq.following_function("sample10:test")
        def sample(message):
            call_history.append("called")
        berrymq.twitter("*:test")
        self.assertEqual(["called"], call_history)

    def _test_composite_condition(self):
        call_history = []
        @berrymq.auto_twitter("sample9_1", entry=True)
        def sample_expose1(arg):
            call_history.append("expose1", entry=True)

        @berrymq.auto_twitter("sample9_2")
        def sample_expose2(arg):
            call_history.append("expose2")

        @berrymq.following_function(berrymq.cond("sample9_1:entry") & \
                                    berrymq.cond("sample9_2:entry"))

        def test_and_condition(message):
            call_history.append("and_condition")

        sample_expose1()
        sample_expose2()
        self.assertEqual(["expose1", "and_condition", "expose2"], call_history)


def test_setup(berrymq_module):
    global berrymq
    berrymq = berrymq_module
    test_suite = unittest.TestSuite()
    test_suite.addTest(unittest.makeSuite(IdentifierTest))
    test_suite.addTest(unittest.makeSuite(TestMessageSendAndReceive))
    return test_suite
